/*  HSYCO.h - HSYCO Library for Arduino
	Copyright (C) 2012-2013 Home Systems Consulting S.p.A. - All rights reserved.

	For information, see the Home Systems Consulting web site:
	http://www.homesystemsconsulting.com/
*/

#ifndef HSYCO_h
#define HSYCO_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#ifdef DACC_INTERFACE
#define HSYC0_DAC0 DAC0
#else
#define HSYC0_DAC0 -1
#endif

#define HSYCO_VERSION   "3.2.0"

extern "C" {
	// callback function
	typedef void (*callbackFunction)(char*, char*);
}

class HSYCOClass {
public:
	HSYCOClass(Stream &s);
	
	// library startup
	void begin();
	void begin(long);
	void begin(Stream &s);
	void debug();
    
	// process input from HSYCO Server
	void process(void);
	
	// send events to HSYCO Server
	void ioSet(char* name, char* value);
	void ioSet(char* name, int value);
	
	// attach and detach callback function
	void attach(callbackFunction myFunction);
	void detach();
    
private:
	Stream &HSYCOSerial;
	
	// input state and buffers
	boolean inputStateStart; // false: waiting for header character; true: header received
	boolean inputStateName; // false: reading name; true: name received, reading value
	char* inputNameBuffer;
	char* inputValueBuffer;
	int inputNameBufferSize;
	int inputValueBufferSize;
	int inputNameBufferLength;
	int inputValueBufferLength;
	boolean debugMode;
	
	// callback function
	callbackFunction currentCallback;
	
	// private methods
	void init(void);
	void inputBufferInit(void);
	void inputBufferReset(void);
	void inputBufferAppend(char c);
	void inputBufferCommit(void);
};

extern HSYCOClass HSYCO;

#endif /* HSYCO_h */