/*  HSYCO Library for Arduino
    Copyright (C) 2012-2013 Home Systems Consulting S.p.A. - All rights reserved.

    For information, see the Home Systems Consulting web site:
    http://www.homesystemsconsulting.com/
	
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
*/

/*
    This sketch reads analog pins A0 to A5 and asynchronously writes the
    corresponding datapoints values to the HSYCO server when a change larger
    than 10 is detected.
*/

#include <HSYCO.h>
#include <limits.h>
#include <stdlib.h>


#define DELTA  10             // the minimum delta that will trigger the execution of ioSet()
#define REFRESHMILLIS  500    // reads analog values approximately every SKIPS milliseconds

int values[6];      // read analog pins A0 to A5
int skipCounter;
char name[] = "pin.a__";


void setup() {
  HSYCO.begin();
  for (int i = 0; i < sizeof(values)/sizeof(int); i++) {
    values[i] = INT_MIN;
  }
}

void loop() {
  HSYCO.process();
  if (skipCounter < REFRESHMILLIS) {
    skipCounter++;
  } else {
    skipCounter = 0;
    for (int i = 0; i < sizeof(values)/sizeof(int); i++) {
      int v = analogRead(A0 + i);
      if (abs(v - values[i]) >= DELTA) {
        values[i] = v;
        itoa(i, name + 5, 10);
        HSYCO.ioSet(name, v);
      }
    }
  }
  delay(1);
}
