/*  HSYCO Library for Arduino
    Copyright (C) 2012-2013 Home Systems Consulting S.p.A. - All rights reserved.

    For information, see the Home Systems Consulting web site:
    http://www.homesystemsconsulting.com/
	
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
*/

/*
    This sketch is a generic Wiegang reader, based on code written by Daniel Smith (www.pagemac.com).
    The Wiegand interface has two data lines, DATA0 and DATA1.  These lines are normall held
    high at 5V.  When a 0 is sent, DATA0 drops to 0V for a few us.  When a 1 is sent, DATA1 drops
    to 0V for a few us.  There is usually a few ms between the pulses.
    Your reader should have at least 4 connections (some readers have more).  Connect the Red wire
    to 5V.  Connect the black to ground.  Connect the green wire (DATA0) to Digital Pin 2 (INT0).
    Connect the white wire (DATA1) to Digital Pin 3 (INT1).
    
    The sketch will write a "wiegand" datapoint with the bit string received on the Wiegand lines, for example:
      IO MONITOR EVENT: arduino.wiegand = 0001
*/

#include <HSYCO.h>

#define MAX_BITS 100               // max number of bits
#define WEIGAND_WAIT_TIME  30 0    // time to wait for another weigand pulse
unsigned char databits[MAX_BITS];  // stores all of the data bits
unsigned char bitCount;            // number of bits currently captured
unsigned char flagDone;            // goes low when data is currently being captured
unsigned int weigand_counter;      // countdown until we assume there are no more bits

// interrupt that happens when INTO goes low (0 bit)
void ISR_INT0() {
  bitCount++;
  flagDone = 0;
  weigand_counter = WEIGAND_WAIT_TIME;  
}

// interrupt that happens when INT1 goes low (1 bit)
void ISR_INT1() {
  databits[bitCount] = 1;
  bitCount++;
  flagDone = 0;
  weigand_counter = WEIGAND_WAIT_TIME;  
}

void setup() {
  HSYCO.begin();

  pinMode(2, INPUT);  // DATA0 (INT0)
  pinMode(3, INPUT);  // DATA1 (INT1)
  // binds the ISR functions to the falling edge of INTO and INT1
  attachInterrupt(0, ISR_INT0, FALLING);
  attachInterrupt(1, ISR_INT1, FALLING);
  weigand_counter = WEIGAND_WAIT_TIME;
}

void loop() {
  HSYCO.process();
  
  // This waits to make sure that there have been no more data pulses before processing data
  if (!flagDone) {
    if (--weigand_counter == 0) {
      flagDone = 1;	
    }
  }
  
  // if we have bits and the weigand counter went out
  if (bitCount > 0 && flagDone) {
    char buffer[bitCount + 1];
    for (unsigned char i = 0; i < bitCount; i++) {
      buffer[i] = '0' + databits[i];
      databits[i] = 0;
    }
    buffer[bitCount] = 0;
    bitCount = 0;
    HSYCO.ioSet("wiegand", buffer);
  }
}
